'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('babel-runtime/regenerator'));
var _extends = _interopDefault(require('babel-runtime/helpers/extends'));
var _asyncToGenerator = _interopDefault(require('babel-runtime/helpers/asyncToGenerator'));
var _JSON$stringify = _interopDefault(require('babel-runtime/core-js/json/stringify'));
var rollupPluginutils = require('rollup-pluginutils');
var compiler = require('vue-component-compiler');
var path = require('path');
var fs = require('fs');
var resolve = _interopDefault(require('resolve'));
var debug = _interopDefault(require('debug'));

var debug$1 = debug('rollup-plugin-vue');

var cache = {};

function s(any) {
    return _JSON$stringify(any, null, 2);
}

function relative(p) {
    if (p.startsWith(process.cwd())) {
        p = p.replace(process.cwd(), '');
        p = p.replace(/^[\\/]/, '');
    }

    return p;
}

function read(file, context) {
    if (fs.lstatSync(context).isFile()) {
        context = path.dirname(context);
    }

    return fs.readFileSync(resolve.sync(file, { basedir: context })).toString();
}

function makeId(id, type, lang, index) {
    return id + '?' + _JSON$stringify({ type: type, index: index }) + '#.' + lang;
}

function vue() {var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    debug$1('Yo! rolling vue!');
    var filter = rollupPluginutils.createFilter(opts.include, opts.exclude);

    delete opts.include;
    delete opts.exclude;

    var vueOptions = {
        hasStyleInjectFn: true,
        isHot: false,
        isServer: opts.server === true || process.env.VUE_ENV === 'server',
        isProduction: opts.production === true || process.env.NODE_ENV === 'production' };


    function parseId(id) {
        if (/\.vue\?\{[^#]+\}#\./.test(id)) {
            var start = id.lastIndexOf('?');
            var end = id.lastIndexOf('#');

            try {
                var parsed = JSON.parse(id.substring(start + 1, end));
                parsed.id = id.substring(0, start);

                return parsed;
            } catch (e) {
                console.error(e);
            }
        }

        if (!filter(id) || id.endsWith('.vue')) return { sfc: true, id: id };

        return { ignore: true };
    }

    return {
        name: 'vue',

        resolveId: function resolveId(request) {
            var q = parseId(request);
            debug$1('Resolve: ' + request);

            if (!q.ignore) return request;

            debug$1('Defer resolve: ' + request);
        },

        load: function load(request) {
            var q = parseId(request);
            debug$1('Load: ' + request);

            if (q.ignore || q.sfc) return;

            debug$1('Part: ' + request);
            var id = q.id;
            var filename = relative(id);
            var descriptor = cache[id];
            // const scopeId = compiler.generateScopeId(relative(filename), process.cwd())

            if (!descriptor) throw Error('SFC (' + filename + ') is not processed.');

            switch (q.type) {
                case 'template':
                    if (!descriptor.template) return '';

                    return descriptor.template.src ? read(descriptor.template.src, id) : descriptor.template.content;
                case 'script':
                    if (!descriptor.script) return 'export default {}\n';

                    return {
                        code: descriptor.script.src ?
                        read(descriptor.script.src, id) :
                        descriptor.script.content,
                        map: descriptor.script.map };

                case 'style':
                    var style = descriptor.styles[q.index];
                    return {
                        code: style.src ?
                        read(style.src, id) :
                        style.content,
                        map: style.map };

                case 'custom':
                    return descriptor.customBlocks[q.index].src ? read(descriptor.customBlocks[q.index].src, id) : descriptor.customBlocks[q.index].content;}

        },

        transform: function transform(source, request) {var _this = this;return _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {var q, filename, scopeId, descriptor, code, output, style, result, _output;return _regeneratorRuntime.wrap(function _callee$(_context) {while (1) {switch (_context.prev = _context.next) {case 0:
                                q = parseId(request);if (!(

                                q.ignore || q.dep)) {_context.next = 3;break;}return _context.abrupt('return');case 3:

                                filename = relative(q.id);
                                scopeId = compiler.generateScopeId(filename, process.cwd());if (!

                                q.sfc) {_context.next = 12;break;}
                                debug$1('  > Compile [component]: ' + q.id);
                                descriptor = compiler.parse(source, filename, {
                                    needsMap: true });


                                cache[q.id] = descriptor;

                                code = compiler.assemble({
                                    script: { id: descriptor.script && makeId(q.id, 'script', descriptor.script.lang || 'js'), descriptor: descriptor.script },
                                    render: { id: descriptor.template && makeId(q.id, 'template', descriptor.template.lang || 'html'), descriptor: descriptor.template },
                                    styles: descriptor.styles.map(
                                    function (style, index) {return { id: makeId(q.id, 'style', style.lang || 'css', index), descriptor: style };}),

                                    customBlocks: descriptor.customBlocks.map(
                                    function (block, index) {return { id: makeId(q.id, 'custom', block.lang || block.type), descriptor: block };}) },

                                filename, _extends({}, vueOptions, { scopeId: scopeId, moduleIdentifier: 'data-v-' + scopeId }));

                                console.log(code);return _context.abrupt('return',

                                { code: code, map: { mappings: '' } });case 12:if (!(


                                q.type === 'template')) {_context.next = 17;break;}
                                debug$1('  > Compile [template]: ' + q.id);
                                output = compiler.compileTemplate({ code: source }, q.id, _extends({}, vueOptions, { scopeId: scopeId }));

                                if (output.map) output.map = { mappings: '' };return _context.abrupt('return',

                                output);case 17:if (!(


                                q.type === 'style')) {_context.next = 27;break;}
                                debug$1('  > Compile [style]: ' + q.id + '#' + q.index);

                                style = cache[q.id].styles[q.index];_context.next = 22;return (
                                    compiler.compileStyle({ code: source, descriptor: style }, q.id, _extends({}, vueOptions, { scopeId: scopeId, async: true })));case 22:result = _context.sent;
                                _output = 'const __default_export__ = ' + (result.modules ? s(result.modules) : 'undefined') + '\n';
                                if (opts.extract) opts.extract(result.code, result.map, result.modules);else
                                {
                                    _output +=
                                    'export const style = ' + s(result.code) + '\n' + ('export const map = ' +
                                    s(result.map) + '\n') + ('const content = [[' +
                                    q.index + ', style, ' + (style.attrs.media && s(style.attrs.media)) + ', ' + (vueOptions.isProduction ? 'undefined' : 'map') + ' ]]\n');
                                    if (vueOptions.isServer) {
                                        _output +=
                                        'import inject from \'vue-component-compiler/src/runtime/inject-style-server\'\n' + '__default_export__ = __default_export__ || {}\n' + ('__default_export__.__inject__ = context => inject(' +

                                        s(q.id) + ', content, ' + vueOptions.isProduction + ', context)\n');
                                    } else {
                                        _output +=
                                        'import inject from \'vue-component-compiler/src/runtime/inject-style-client\'\n' + ('inject(' +
                                        s(q.id) + ', content, ' + vueOptions.isProduction + ')\n');
                                    }
                                }
                                _output += 'export default __default_export__';return _context.abrupt('return',

                                { code: _output, map: { mappings: '' } });case 27:case 'end':return _context.stop();}}}, _callee, _this);}))();

        } };

}

module.exports = vue;
